@@ -10,10 +10,12 @@ import logging
 import re
 import os
 import struct
+from datetime import datetime
 from pathlib import Path
 
 from .const import (
     ControlType,
+    DriveName,
     Login,
     MemoryType,
     LSV2Err,
@@ -71,6 +73,7 @@ class LSV2:
                 ParCCC.SET_BUF3072,
                 ParCCC.SET_BUF4096,
                 ParCCC.SECURE_FILE_SEND,
+                ParCCC.SCREENDUMP,
             )
         else:
             logging.info(
@@ -1165,6 +1168,9 @@ class LSV2:
         :returns: True or False if command was executed successfully
         :rtype: bool
         """
+        if not self.login(Login.MONITOR):
+            logging.error("clould not log in as user MONITOR")
+            return False
 
         payload = bytearray()
         if unlocked:
@@ -1253,6 +1259,10 @@ class LSV2:
         :returns: True or False if command was executed successfully
         :rtype: bool
         """
+        if not self.login(Login.MONITOR):
+            logging.error("clould not log in as user MONITOR")
+            return False
+
         payload = bytearray()
         payload.extend(struct.pack("!H", key_code))
 
@@ -1359,7 +1369,9 @@ class LSV2:
                 or entry["Name"].endswith(":")
             ):
                 continue
-            current_fs_element = str(current_path + entry["Name"]).replace("/", PATH_SEP)
+            current_fs_element = str(current_path + entry["Name"]).replace(
+                "/", PATH_SEP
+            )
             if entry["is_directory"] is True and descend is True:
                 if self.change_directory(current_fs_element):
                     content.extend(self._walk_dir())
@@ -1486,3 +1498,34 @@ class LSV2:
 
         logging.error("an error occurred while querying axes position")
         return False
+
+    def grab_screen_dump(self, image_path: Path):
+        """create screen_dump of current control screen and save it as bitmap"""
+        if not self.login(Login.FILETRANSFER):
+            logging.error("clould not log in as user FILE")
+            return False
+
+        temp_file_path = (
+            DriveName.TNC
+            + PATH_SEP
+            + "screendump_"
+            + datetime.now().strftime("%Y%m%d_%H%M%S")
+            + ".bmp"
+        )
+
+        if not self.set_system_command(ParCCC.SCREENDUMP, temp_file_path):
+            logging.error("screen dump was not created")
+            return False
+
+        if not self.recive_file(
+            remote_path=temp_file_path, local_path=image_path, binary_mode=True
+        ):
+            logging.error("could not download screen dump from control")
+            return False
+
+        if not self.delete_file(temp_file_path):
+            logging.error("clould not delete temporary file on control")
+            return False
+
+        logging.debug("successfully recived screen dump")
+        return True
